@@ -6,8 +6,9 @@
 ## Created: 8/2013
 ## Updated: 10/30/2013
 
-pedgene.stats <- function(geno, c.factor, chrom, male.dose, sex,
-                          resid, weights=NULL, acc.davies=1e-6) {
+pedgene.stats <- function(geno, c.factor, chrom, male.dose, sex, resid,
+                          weights=NULL, weights.mb=FALSE, weights.beta=c(1,25),
+                          method="davies", acc.davies=1e-6) {
 
     x.chrom <- as.character(chrom)=="X"
     
@@ -20,8 +21,7 @@ pedgene.stats <- function(geno, c.factor, chrom, male.dose, sex,
     nvariant <- ncol(geno)
     nvariant.noninform <- nvariant0 - nvariant
     if(nvariant==0) {
-        return(list(stat.kernel = NA, df.kernel=NA,
-                    pval.kernel = NA,  pval.kernel.davies = NA,
+        return(list(stat.kernel = NA, pval.kernel = NA,  
                     stat.burden = NA, pval.burden = NA,
                     nvariant = nvariant, noninform=nvariant.noninform))
       }
@@ -54,21 +54,22 @@ pedgene.stats <- function(geno, c.factor, chrom, male.dose, sex,
           }
       }
    
-    # Madsen-Browning weights
-    if(x.chrom)
-      {
-        count.male   <- apply(geno[sex==1,], 2, sum)
-        count.female <- apply(geno[sex==2,], 2, sum)
-        n.male   <- sum(sex==1)
-        n.female <- sum(sex==2)
-        maf <- (count.male + count.female)/(n.male + 2*n.female)
-      } else
-    {
+    if(x.chrom) {
+      count.male   <- apply(geno[sex==1,,drop=FALSE], 2, sum)
+      count.female <- apply(geno[sex==2,,drop=FALSE], 2, sum)
+      n.male   <- sum(sex==1)
+      n.female <- sum(sex==2)
+      maf <- (count.male + count.female)/(n.male + 2*n.female)
+    } else {
       maf <- apply(geno/2, 2, mean)
-     }
-
-    if(is.null(weights)) {
-      wt <- 1/sqrt(maf * (1-maf))
+    }   
+    if(is.null(weights)) {    
+      if(weights.mb==TRUE) {        
+        wt <- 1/sqrt(maf * (1-maf))
+      } else {
+        ## weights=NULL and weights.MB=FALSE, all default to do weights.beta
+        wt <- dbeta(maf, weights.beta[1], weights.beta[2])
+      }
     } else {
       wt <- weights[ v > 0 ]
     }
@@ -76,75 +77,66 @@ pedgene.stats <- function(geno, c.factor, chrom, male.dose, sex,
     # estimate cor among markers
     r.mat <- cor(geno)
    
-    ## compute f from top of p412.  results in f=1 in MD weights,
-    ## left here in case other weights used
+    ## compute f from top of p412.  results in f=1 in M-B weights,
+    ## still needed for beta and user-specified weights, b/c it is binomial variance
     
     f <- wt * sqrt(maf*(1-maf))
     
     fRmat <- (f %o% f) * r.mat
 
-    var.z <- fRmat * c.factor
-    
-    e.Q <- sum(diag(var.z))
-    var.Q <- 2*sum(diag(var.z %*% var.z))
+    var.z <- fRmat * c.factor    
     
     # score males according to male.dose
-    if(x.chrom & (male.dose !=1) )
-      {
-        geno.score <- geno
-       
-        geno.score[sex==1,] <- geno[sex==1,]*male.dose
-
-
-        # kernel stat info
-        kmat <- geno.score %*% diag(wt^2) %*% t(geno.score)
-
-        # Burden stat info
-        burden.score.subject <- as.vector(geno.score %*% wt)
-        
-      } else
-    {
-     
-      # kernel stat info
+    if(x.chrom & (male.dose !=1) )  {
+      geno.score <- geno     
+      geno.score[sex==1,] <- geno[sex==1,]*male.dose       
+      ## kernel stat info
+      kmat <- geno.score %*% diag(wt^2,nrow=length(wt),ncol=length(wt)) %*% t(geno.score)
+      ## Burden stat info
+      burden.score.subject <- as.vector(geno.score %*% wt)        
+    } else {     
+      ## kernel stat info
       kmat <- geno %*% diag(wt^2,nrow=length(wt),ncol=length(wt)) %*% t(geno)
-
-      # Burden stat info
+      ## Burden stat info
       burden.score.subject <- as.vector(geno %*% wt)
     }
 
     ########### Burden stat (2-sided)     
-    stat.num <- (resid %*% burden.score.subject)^2
-    factor.sum <- sum(fRmat)    
-    stat.denom <- factor.sum * c.factor
-    
+    stat.num <- (resid %*% burden.score.subject)
+    factor.sum <- sum(fRmat)
+    stat.denom <- sqrt(factor.sum * c.factor)    
     burden.stat <- stat.num / stat.denom
-    burden.pval <- pchisq(burden.stat, 1,lower.tail=FALSE)
-   
+    burden.pval <- pchisq(burden.stat^2, 1,lower.tail=FALSE)
+    
     ########## Quadratic kernel stat
-    # if only 1 variant, reduces to burden stat
+    ## if only 1 variant, reduces to burden stat, o/w do kernel test
     if(nvariant>1) {
       stat.kernel <- as.vector(resid %*% kmat %*% resid)
-      
-      scale <- 2*e.Q / var.Q
-      df    <- 2*e.Q^2 / var.Q
-      pval.kernel <- pchisq(stat.kernel*scale, df=df, lower.tail=FALSE)
-      
       eig <-eigen(var.z, symmetric=T, only.values=T)
-      evals <-eig$values[eig$values>1e-6*eig$values[1]] 
-      pval.kernel.davies <- davies(stat.kernel , evals, acc=acc.davies)$Qq
-      ## Davies' method sometimes instable, returns out-of-range p-value
-      ## set to NA
-      if( (pval.kernel.davies > 1.0) | (pval.kernel.davies < 0.0) ) {
-        pval.kernel.davies <- NA
+      evals <-eig$values[eig$values>1e-6*eig$values[1]]
+      if(method=="davies") {
+        pval.kernel <- davies(stat.kernel , evals, acc=acc.davies)$Qq        
+        ## Davies' method sometimes instable, returns out-of-range p-value
+        ## Help file suggests lerge acc.davies of 1e-4 or so to fix
       }
+      if(method=="kounen") {
+        ## in main method, require(survery) for kounen
+        pval.kernel <- pchisqsum(stat.kernel, rep(1, length(evals)),
+                           evals, lower.tail=FALSE, method="saddle")
+      }
+
+      ## davies and kounen sometimes return out of range p-values.
+      ## fix to 1 or 0 on either side.
+      pval.kernel <- min(pval.kernel, 1)
+      pval.kernel <- max(pval.kernel,0)
+     
     } else {
-      stat.kernel <- burden.stat  
-      pval.kernel <- pval.kernel.davies <- burden.pval
+      stat.kernel <- burden.stat^2  
+      pval.kernel <- burden.pval
     }     
     
-    lst <- list(stat.kernel = stat.kernel, df.kernel=df,
+    lst <- list(stat.kernel = stat.kernel, 
                      pval.kernel = pval.kernel,
-		     pval.kernel.davies = pval.kernel.davies,
                      stat.burden = burden.stat,
                      pval.burden = burden.pval,
                      nvariant = nvariant,
