@@ -1,11 +1,24 @@
 
 ## Expanded test script for pedgene package
+devel=FALSE
+if(devel) {
+  rfiles <- list.files(path="../R/", pattern="*.R$")
+  for(f in rfiles) source(paste("../R/",f,sep=""))
+  dfiles <- list.files(path="../data/", pattern="*.RData$")
+  for(d in dfiles) load(paste("../data/",d,sep=""))
+  library(kinship2)
+  library(survey)
+} else {
+  
+  require(pedgene)
+  data(example.ped)
+  data(example.geno)
+  data(example.map)
+  
+}
 
-require(pedgene)
+#require(survery)
 
-data(example.ped)
-data(example.geno)
-data(example.map)
 
 ######################################
 ## From Dan Weeks, issues to check
@@ -30,9 +43,9 @@ data(example.map)
 #$pval.burden
 #[1,] 0.02692495
 
-## base case
+## base case, m-b weights
 pg.out.m2 <- pedgene(ped=example.ped, geno=example.geno, map=example.map, male.dose=2,
-                     weights=NULL, checkpeds=TRUE)
+                     weights.mb=TRUE,checkpeds=TRUE)
 
 # summary/print and plot methods for this object
 print.pedgene(pg.out.m2,digits=3)
@@ -41,9 +54,20 @@ print.pedgene(pg.out.m2,digits=3)
 ## AA   AA     1        80.1       0.404        4.90     0.02692
 ## AX   AX     X       198.2       0.186        7.82     0.00515
 
+## base case, beta weights, no pedcheck
+pg.beta.m2 <- pedgene(ped=example.ped, geno=example.geno, map=example.map, male.dose=2, verbose.return=TRUE)
+names(pg.beta.m2)
+lapply(pg.beta.m2$save, dim)
+
+print(pg.beta.m2, digits=4)
+
+## base case, mb weights, method=kounen, no pedcheck
+pg.kounen.m2 <- pedgene(ped=example.ped, geno=example.geno, map=example.map, male.dose=2, weights.mb=TRUE,method="kounen")
+print(pg.kounen.m2,digits=4)
+
 ## try making ped1 disconeeded by taking 2nd-generation parents away
 ## results will differ a little
-pg.out.m2.rm34 <- pedgene(example.ped[-(3:4),], example.geno, example.map, male.dose=2, checkpeds=FALSE)
+pg.out.m2.rm34 <- pedgene(example.ped[-(3:4),], example.geno, example.map, male.dose=2, checkpeds=FALSE, weights.mb=TRUE)
 pg.out.m2.rm34
 
 ## Test character ids, which is robust now because we're now making super-ids by
@@ -72,12 +96,18 @@ geno.narow[4,3:ncol(example.geno)] <- NA
 # to check if male dose>1 for males on X chrom -- works 
 #geno.narow[3,3:ncol(example.geno)] <- ifelse(geno.narow[3,2:ncol(example.geno)]==0,0,2)
 pg.narow.m2 <- pedgene(ped=example.ped, geno=geno.narow, map=example.map, male.dose=2,
-                     weights=NULL, checkpeds=TRUE)
+                     weights=NULL, weights.mb=TRUE,checkpeds=TRUE)
 print(pg.narow.m2,digits=3)
 
+## choose marker 4 as the 1-snp to represent the gene
+## single-snp genes reduce kernel test to burden, check that p-vals agree, stat.kern=stat.burd^2
+## This also caused indexing problems in the past.
+pg.g1.m2 <- pedgene(ped=example.ped, geno=example.geno[,c(1:2,6,13:22)],
+                    map=example.map[c(1,11:20),], male.dose=2,weights.mb=TRUE)
+pg.g1.m2
 
 # male dose=1
-pg.out.m1 <- pedgene(example.ped, example.geno, example.map, male.dose=1)
+pg.out.m1 <- pedgene(example.ped, example.geno, example.map, male.dose=1, )
 
 print(pg.out.m1, digits=3)
 ##    gene chrom stat.kernel pval.kernel stat.burden pval.burden
@@ -94,11 +124,13 @@ pg.out.nomap
 
 ## Testing first gene with dose=2-dose
 geno.recode <- cbind(example.geno[,1:2], 2-example.geno[,grep("AA", names(example.geno))])
-pg.recode <- pedgene(example.ped, geno.recode, male.dose=2)
-
+pg.recode.mb <- pedgene(example.ped, geno.recode, male.dose=2, weights.mb=TRUE)
 ## note when map not given, assumes all 1 gene, and assigns "unknown" gene/chrom
-pg.recode
+pg.recode.mb
 
+pg.recode.beta <- pedgene(example.ped, geno.recode, male.dose=2)
+## note when map not given, assumes all 1 gene, and assigns "unknown" gene/chrom
+pg.recode.beta
 
 
 ## weights, Madsen-Browning
@@ -131,8 +163,8 @@ genobig <- cbind(genobig, example.geno[,6], rep(1, nrow(example.geno)), rep(2, n
 mapbig <- rbind(mapbig, c(10, "onevar"), c(11,"novar"), c(11, "novar"))
                 
 
-pgbig.m2 <- pedgene(example.ped, genobig, mapbig, male.dose=2)
-pgbig.m1 <- pedgene(example.ped, genobig, mapbig, male.dose=1)
+pgbig.m2 <- pedgene(example.ped, genobig, mapbig, male.dose=2, acc.davies=1e-4)
+pgbig.m1 <- pedgene(example.ped, genobig, mapbig, male.dose=1, acc.davies=1e-4)
 
 print(pgbig.m2, digits=3)
 print(pgbig.m1, digits=3)
